@@ -10,7 +10,8 @@ from tempfile import gettempdir
 
 import pytest
 
-from hdx.utilities.downloader import Download, DownloadError
+from hdx.utilities.base_downloader import DownloadError
+from hdx.utilities.downloader import Download
 from hdx.utilities.session import SessionError
 from hdx.utilities.useragent import UserAgent
 
@@ -45,7 +46,11 @@ class TestDownloader:
 
     @pytest.fixture(scope="class")
     def fixtureurl(self):
-        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/master/tests/fixtures/test_data.csv"
+        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/main/tests/fixtures/test_data.csv"
+
+    @pytest.fixture(scope="class")
+    def fixtureurlexcel(self):
+        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/base_downloader/tests/fixtures/downloader/test_data.xlsx"
 
     @pytest.fixture(scope="class")
     def getfixtureurl(self):
@@ -57,15 +62,15 @@ class TestDownloader:
 
     @pytest.fixture(scope="class")
     def fixturenotexistsurl(self):
-        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/master/tests/fixtures/NOTEXIST.csv"
+        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/main/tests/fixtures/NOTEXIST.csv"
 
     @pytest.fixture(scope="class")
     def fixtureprocessurl(self):
-        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/master/tests/fixtures/downloader/test_csv_processing.csv?a=1"
+        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/main/tests/fixtures/downloader/test_csv_processing.csv?a=1"
 
     @pytest.fixture(scope="class")
     def fixtureprocessurlblank(self):
-        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/master/tests/fixtures/downloader/test_csv_processing_blanks.csv?a=1"
+        return "https://raw.githubusercontent.com/OCHA-DAP/hdx-python-utilities/main/tests/fixtures/downloader/test_csv_processing_blanks.csv?a=1"
 
     def test_get_path_for_url(
         self, tmpdir, fixtureurl, configfolder, downloaderfolder
@@ -412,9 +417,9 @@ class TestDownloader:
         with pytest.raises(DownloadError), Download() as downloader:
             downloader.download(fixturenotexistsurl)
         with Download() as downloader:
-            downloader.download(fixtureurl)
+            text = downloader.download_text(fixtureurl)
             assert downloader.get_status() == 200
-            assert len(downloader.get_headers()) == 24
+            assert len(downloader.get_headers()) in (24, 25)
             assert (
                 bool(
                     re.match(r"7\d\d", downloader.get_header("Content-Length"))
@@ -422,7 +427,7 @@ class TestDownloader:
                 is True
             )
             assert (
-                downloader.get_text()
+                text
                 == """GWNO,EVENT_ID_CNTY,EVENT_ID_NO_CNTY,EVENT_DATE,YEAR,TIME_PRECISION,EVENT_TYPE,ACTOR1,ALLY_ACTOR_1,INTER1,ACTOR2,ALLY_ACTOR_2,INTER2,INTERACTION,COUNTRY,ADMIN1,ADMIN2,ADMIN3,LOCATION,LATITUDE,LONGITUDE,GEO_PRECISION,SOURCE,NOTES,FATALITIES
 615,1416RTA,,18/04/2001,2001,1,Violence against civilians,Police Forces of Algeria (1999-),,1,Civilians (Algeria),Berber Ethnic Group (Algeria),7,17,Algeria,Tizi Ouzou,Beni-Douala,,Beni Douala,36.61954,4.08282,1,Associated Press Online,A Berber student was shot while in police custody at a police station in Beni Douala. He later died on Apr.21.,1
 615,2229RTA,,19/04/2001,2001,1,Riots/Protests,Rioters (Algeria),Berber Ethnic Group (Algeria),5,Police Forces of Algeria (1999-),,1,15,Algeria,Tizi Ouzou,Tizi Ouzou,,Tizi Ouzou,36.71183,4.04591,3,Kabylie report,"Riots were reported in numerous villages in Kabylie, resulting in dozens wounded in clashes between protesters and police and significant material damage.",0
@@ -441,12 +446,12 @@ class TestDownloader:
                     [("b", "4"), ("d", "3"), ("id", "10"), ("lala", "a")]
                 ).items()
             )
-            downloader.download(
+            json = downloader.download_json(
                 f"{postfixtureurl}?id=3&lala=b",
                 post=True,
                 parameters=OrderedDict([("a", "3"), ("c", "2")]),
             )
-            assert list(downloader.get_json()["form"].items()) == list(
+            assert list(json["form"].items()) == list(
                 OrderedDict(
                     [("a", "3"), ("c", "2"), ("id", "3"), ("lala", "b")]
                 ).items()
@@ -462,8 +467,8 @@ class TestDownloader:
                     [("b", "4"), ("d", "3"), ("id", "10"), ("lala", "a")]
                 ).items()
             )
-            downloader.download(fixturefile)
-            assert list(downloader.get_yaml()["mykey"].items()) == list(
+            yaml = downloader.download_yaml(fixturefile)
+            assert list(yaml["mykey"].items()) == list(
                 OrderedDict(
                     [
                         ("param1", "value 1"),
@@ -474,14 +479,27 @@ class TestDownloader:
                 ).items()
             )
 
-    def test_download_tabular_key_value(self, fixtureurl, fixtureprocessurl):
+    def test_download_tabular_key_value(
+        self, fixtureurl, fixtureurlexcel, fixtureprocessurl
+    ):
         with Download() as downloader:
             result = downloader.download_tabular_key_value(
-                fixtureurl, file_type="csv"
+                fixtureurl,
+                file_type="csv",
+                delimiter=",",
+                skip_initial_space=True,
+                infer_types=True,
             )
-            assert result == {"615": "2231RTA", "GWNO": "EVENT_ID_CNTY"}
+            assert result == {615: "2231RTA", "GWNO": "EVENT_ID_CNTY"}
             result = downloader.download_tabular_key_value(
-                fixtureprocessurl, headers=2
+                fixtureurlexcel,
+                file_type="xlsx",
+                sheet="test_data",
+                infer_types=True,
+            )
+            assert result == {615: "2231RTA", "GWNO": "EVENT_ID_CNTY"}
+            result = downloader.download_tabular_key_value(
+                fixtureprocessurl, headers=3
             )
             assert result == {"coal": "3", "gas": "2"}
             with pytest.raises(DownloadError):
@@ -489,22 +507,34 @@ class TestDownloader:
                     "NOTEXIST://NOTEXIST.csv"
                 )
 
-    @staticmethod
-    def fix_strings(
-        result,
-    ):  # This isn't needed when I run locally but is when run in Travis!
-        for x in result:
-            for y in result[x]:
-                result[x][y] = result[x][y].replace("'", "")
-
     def test_get_tabular_rows_as_list(self, fixtureprocessurl):
         with Download() as downloader:
-            rows = list(downloader.get_tabular_rows_as_list(fixtureprocessurl))
+            headers, rows = downloader.get_tabular_rows_as_list(
+                fixtureprocessurl
+            )
+            rows = list(rows)
             assert rows == [
                 ["la1", "ha1", "ba1", "ma1"],
                 ["header1", "header2", "header3", "header4"],
-                ["coal", "3", "7.4", "needed"],
-                ["gas", "2", "6.5", "n/a"],
+                ["coal", "3", "7.4", "'needed'"],
+                ["gas", "2", "6.5", "'n/a'"],
+            ]
+
+    def test_get_tabular_rows_as_dict(self, fixtureprocessurl):
+        with Download() as downloader:
+            headers, rows = downloader.get_tabular_rows_as_dict(
+                fixtureprocessurl
+            )
+            rows = list(rows)
+            assert rows == [
+                {
+                    "la1": "header1",
+                    "ha1": "header2",
+                    "ba1": "header3",
+                    "ma1": "header4",
+                },
+                {"la1": "coal", "ha1": "3", "ba1": "7.4", "ma1": "'needed'"},
+                {"la1": "gas", "ha1": "2", "ba1": "6.5", "ma1": "'n/a'"},
             ]
 
     def test_get_tabular_rows(self, fixtureprocessurl, fixtureprocessurlblank):
@@ -512,8 +542,8 @@ class TestDownloader:
             expected = [
                 ["la1", "ha1", "ba1", "ma1"],
                 ["header1", "header2", "header3", "header4"],
-                ["coal", "3", "7.4", "needed"],
-                ["gas", "2", "6.5", "n/a"],
+                ["coal", "3", "7.4", "'needed'"],
+                ["gas", "2", "6.5", "'n/a'"],
             ]
             expected_headers = expected[0]
             headers, iterator = downloader.get_tabular_rows(fixtureprocessurl)
@@ -524,14 +554,14 @@ class TestDownloader:
             )
             assert headers == expected_headers
             blank_expected = copy.deepcopy(expected[1:])
-            blank_expected[2][0] = ""
+            blank_expected[2][0] = None
             assert list(iterator) == blank_expected
             headers, iterator = downloader.get_tabular_rows(
                 fixtureprocessurlblank, ignore_blank_rows=False
             )
             assert headers == expected_headers
-            blank_expected.append(list())
-            blank_expected.insert(1, list())
+            blank_expected.append([None, None, None, None])
+            blank_expected.insert(1, [None, None, None, None])
             assert list(iterator) == blank_expected
             headers, iterator = downloader.get_tabular_rows(
                 fixtureprocessurl, headers=1
@@ -570,8 +600,8 @@ class TestDownloader:
                     "ba1": "header3",
                     "ma1": "header4",
                 },
-                {"la1": "coal", "ha1": "3", "ba1": "7.4", "ma1": "needed"},
-                {"la1": "gas", "ha1": "2", "ba1": "6.5", "ma1": "n/a"},
+                {"la1": "coal", "ha1": "3", "ba1": "7.4", "ma1": "'needed'"},
+                {"la1": "gas", "ha1": "2", "ba1": "6.5", "ma1": "'n/a'"},
             ]
             assert list(iterator) == expected_dicts
             headers, iterator = downloader.get_tabular_rows(
@@ -579,7 +609,7 @@ class TestDownloader:
             )
             assert headers == expected[2]
             expected_dicts = [
-                {"coal": "gas", "3": "2", "7.4": "6.5", "needed": "n/a"}
+                {"coal": "gas", "3": "2", "7.4": "6.5", "'needed'": "'n/a'"}
             ]
             assert list(iterator) == expected_dicts
 
@@ -593,9 +623,9 @@ class TestDownloader:
                 header_insertions=[(2, "la")],
                 row_function=testfn,
             )
-            expected_headers = ["coal", "3", "la", "7.4", "needed"]
+            expected_headers = ["coal", "3", "la", "7.4", "'needed'"]
             assert headers == expected_headers
-            assert list(iterator) == [["gas", "2", "lala", "6.5", "n/a"]]
+            assert list(iterator) == [["gas", "2", "lala", "6.5", "'n/a'"]]
 
             def testfn(headers, row):
                 row["la"] = "lala"
@@ -631,22 +661,34 @@ class TestDownloader:
                     "ma1": "header4",
                     "la": "lala",
                 },
-                {"la": "lala"},
+                {
+                    "la1": None,
+                    "ha1": None,
+                    "ba1": None,
+                    "ma1": None,
+                    "la": "lala",
+                },
                 {
                     "la1": "coal",
                     "ha1": "3",
                     "ba1": "7.4",
-                    "ma1": "needed",
+                    "ma1": "'needed'",
                     "la": "lala",
                 },
                 {
-                    "la1": "",
+                    "la1": None,
                     "ha1": "2",
                     "ba1": "6.5",
-                    "ma1": "n/a",
+                    "ma1": "'n/a'",
+                    "la": "lala",
+                },
+                {
+                    "la1": None,
+                    "ha1": None,
+                    "ba1": None,
+                    "ma1": None,
                     "la": "lala",
                 },
-                {"la": "lala"},
             ]
             assert list(iterator) == expected_dicts
             headers, iterator = downloader.get_tabular_rows(
@@ -696,25 +738,23 @@ class TestDownloader:
             result = downloader.download_tabular_rows_as_dicts(
                 fixtureprocessurl, headers=2
             )
-            self.fix_strings(result)
             assert result == {
                 "coal": {
                     "header2": "3",
                     "header3": "7.4",
-                    "header4": "needed",
+                    "header4": "'needed'",
                 },
-                "gas": {"header2": "2", "header3": "6.5", "header4": "n/a"},
+                "gas": {"header2": "2", "header3": "6.5", "header4": "'n/a'"},
             }
             result = downloader.download_tabular_rows_as_dicts(
                 fixtureprocessurl, headers=2, keycolumn=2
             )
-            self.fix_strings(result)
             assert result == {
-                "2": {"header1": "gas", "header3": "6.5", "header4": "n/a"},
+                "2": {"header1": "gas", "header3": "6.5", "header4": "'n/a'"},
                 "3": {
                     "header1": "coal",
                     "header3": "7.4",
-                    "header4": "needed",
+                    "header4": "'needed'",
                 },
             }
 
@@ -723,20 +763,18 @@ class TestDownloader:
             result = downloader.download_tabular_cols_as_dicts(
                 fixtureprocessurl, headers=2
             )
-            self.fix_strings(result)
             assert result == {
                 "header2": {"coal": "3", "gas": "2"},
                 "header3": {"coal": "7.4", "gas": "6.5"},
-                "header4": {"coal": "needed", "gas": "n/a"},
+                "header4": {"coal": "'needed'", "gas": "'n/a'"},
             }
             result = downloader.download_tabular_cols_as_dicts(
                 fixtureprocessurl, headers=2, keycolumn=2
             )
-            self.fix_strings(result)
             assert result == {
                 "header1": {"3": "coal", "2": "gas"},
                 "header3": {"3": "7.4", "2": "6.5"},
-                "header4": {"3": "needed", "2": "n/a"},
+                "header4": {"3": "'needed'", "2": "'n/a'"},
             }
 
     def test_get_column_positions(self):
@@ -745,3 +783,24 @@ class TestDownloader:
             "b": 1,
             "c": 2,
         }
+
+    @staticmethod
+    def assert_downloaders(downloader1, downloader2, params):
+        user_agent, custom_user_agent, extra_params_dict = params
+        test_url = "http://www.lalala.com/lala"
+        assert downloader1.session.headers["User-Agent"].endswith(user_agent)
+        assert downloader1.session.auth is None
+        assert downloader1.get_full_url(test_url) == test_url
+        assert downloader2.session.headers["User-Agent"].endswith(
+            custom_user_agent
+        )
+        assert downloader2.session.auth == ("user", "pass")
+        key, value = next(iter(extra_params_dict.items()))
+        assert (
+            downloader2.get_full_url(test_url) == f"{test_url}?{key}={value}"
+        )
+
+    def test_generate_downloaders(self, downloaders):
+        downloader1 = Download.get_downloader()
+        downloader2 = Download.get_downloader("test")
+        self.assert_downloaders(downloader1, downloader2, downloaders)
